@@ -1,3 +1,11 @@
+
+"""
+Adapter for Google BigQuery database
+"""
+
+
+from typing import Optional
+
 import gzip
 import os
 import os.path
@@ -59,11 +67,38 @@ def _bigquery_is_bad_expr(dbmodel, expression):
     )
 
 
+def _bigquery_any_expr(dbmodel, expression):
+    subexpr = dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
+    return f'LOGICAL_OR({subexpr})'
+
+
+def _bigquery_all_expr(dbmodel, expression):
+    subexpr = dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
+    return f'LOGICAL_AND({subexpr})'
+
+
+def _bigquery_any_value_expr(dbmodel, expression):
+    return 'ANY_VALUE(' + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False) + ')'
+
+
+def _bigquery_ieee_divide_expr(dbmodel, expression):
+    # don't issue an error
+    # https://cloud.google.com/bigquery/docs/reference/standard-sql/mathematical_functions#ieee_divide
+    assert len(expression.args) == 2
+    e0 = dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
+    e1 = dbmodel.expr_to_sql(expression.args[1], want_inline_parens=True)
+    return f'IEEE_DIVIDE({e0}, 1.0 * {e1})'
+
+
 BigQuery_formatters = {
     "median": _bigquery_median_expr,
     "is_bad": _bigquery_is_bad_expr,
     "std": _bigquery_std_expr,
     "var": _bigquery_var_expr,
+    "any": _bigquery_any_expr,
+    "all": _bigquery_all_expr,
+    "any_value": _bigquery_any_value_expr,
+    "%/%": _bigquery_ieee_divide_expr,
 }
 
 
@@ -71,7 +106,7 @@ class BigQueryModel(data_algebra.db_model.DBModel):
     """A model of how SQL should be generated for BigQuery
        connection should be google.cloud.bigquery.client.Client"""
 
-    def __init__(self, *, table_prefix=None):
+    def __init__(self, *, table_prefix: Optional[str] = None):
         data_algebra.db_model.DBModel.__init__(
             self,
             identifier_quote="`",
