@@ -649,14 +649,13 @@ db_expr_formatters = {
 db_default_op_replacements = {
     "==": "=",
     "cumsum": "SUM",
+    "cumcount": "COUNT",
+    "cummax": "MAX",
+    "cummin": "MIN",
+    "cumprod": "PROD",
+    "cummean": "AVG",
     "and": "AND",
-    "&": "AND",
-    "&&": "AND",
     "or": "OR",
-    "|": "OR",
-    "||": "OR",
-    "!": "NOT",
-    "~": "NOT",
     "_count": "COUNT",
     "_ngroup": "NGROUP",
     "_row_number": "ROW_NUMBER",
