@@ -72,6 +72,8 @@ fn_names_that_imply_windowed_situation = {
     "ohlc",
     "pct_change",
     "rank",
+    "_row_number",
+    "row_number",
     "shift",
     "size",
     "std",
@@ -94,6 +96,8 @@ fn_names_that_imply_ordered_windowed_situation = {
     "shift",
     "lag",
     "lead",
+    "_row_number",
+    "row_number",
 }
 
 
@@ -101,7 +105,7 @@ fn_names_that_imply_ordered_windowed_situation = {
 fn_names_not_allowed_in_project = {
     "ngroup",
     "_ngroup",
-}
+}.union(fn_names_that_imply_ordered_windowed_situation)
 
 
 # fns that don't have consistent windowed implementations we want to support
@@ -282,14 +286,24 @@ class Term(PreTerm, abc.ABC):
                 )
         return Expression(op, (self, other), inline=inline, method=method)
 
-    def __rop_expr__(self, op, other):
+    def __rop_expr__(self, op, other, *,  check_types=True):
         """reversed binary expression"""
+        inline = True
+        method = False
         assert isinstance(op, str)
         if not isinstance(other, Term):
             other = enc_value(other)
         assert not _is_none_value(self)
         assert not _is_none_value(other)
-        return Expression(op, (other, self), inline=True)
+        if check_types:
+            obvious_type_problem = _check_expr_incompatible_types(self, other)
+            if obvious_type_problem is not None:
+                raise TypeError(
+                    f"trying to combine incompatible values:"
+                    + f" {other}:{obvious_type_problem[0]}"
+                    + f" and {self}:{obvious_type_problem[0]} with {op}"
+                )
+        return Expression(op, (other, self), inline=inline, method=method)
 
     def __uop_expr__(self, op, *, params=None, inline=False):
         """unary expression"""
@@ -305,8 +319,6 @@ class Term(PreTerm, abc.ABC):
         if not isinstance(y, Term):
             y = enc_value(y)
         assert not _is_none_value(self)
-        assert not _is_none_value(x)
-        assert not _is_none_value(y)
         return Expression(op, (self, x, y), inline=inline, method=method)
 
     # try to get at == and other comparison operators
@@ -355,6 +367,9 @@ class Term(PreTerm, abc.ABC):
     def __rtruediv__(self, other):
         return self.__rop_expr__("/", other)
 
+    def float_divide(self, other):
+        return self.__op_expr__("%/%", other)
+
     def __floordiv__(self, other):
         return self.__op_expr__("//", other)
 
@@ -669,6 +684,18 @@ class Term(PreTerm, abc.ABC):
         """
         return self.__uop_expr__("is_monotonic_increasing")
 
+    def any_value(self):
+        """
+        Return any_value (vectorized).
+        """
+        return self.__uop_expr__("any_value")
+
+    def first(self):
+        """
+        Return first (vectorized).
+        """
+        return self.__uop_expr__("first")
+
     def last(self):
         """
         Return last (vectorized).
@@ -677,7 +704,7 @@ class Term(PreTerm, abc.ABC):
 
     def max(self):
         """
-        Return last (vectorized).
+        Return max (vectorized).
         """
         return self.__uop_expr__("max")
 
@@ -737,7 +764,10 @@ class Term(PreTerm, abc.ABC):
         """
         if periods is None:
             periods = Value(1)
-        assert isinstance(periods, Value)  # digits control
+        assert isinstance(periods, Value)
+        assert isinstance(periods.value, int)
+        if periods.value == 0:
+            raise ValueError("0-period shift not allowed")
         return self.__op_expr__("shift", periods, inline=False, method=True)
 
     # our ad-hoc definitions
@@ -748,6 +778,18 @@ class Term(PreTerm, abc.ABC):
         """
         return self.__uop_expr__("is_null")
 
+    def is_nan(self):
+        """
+        Return which items are nan (vectorized).
+        """
+        return self.__uop_expr__("is_nan")
+
+    def is_inf(self):
+        """
+        Return which items are inf (vectorized).
+        """
+        return self.__uop_expr__("is_inf")
+
     def is_bad(self):
         """
         Return which items are bad (null or nan) (vectorized).
@@ -757,10 +799,21 @@ class Term(PreTerm, abc.ABC):
     def if_else(self, x, y):
         """
         Vectorized selection between two argument vectors.
+        if_else(True, 1, 2) > 1, if_else(False, 1, 2) -> 2.
+        None propagating behavior if_else(None, 1, 2) -> None.
         """
         # could check if x and y are compatible types
         return self.__triop_expr__("if_else", x, y, method=True)
 
+    def where(self, x, y):
+        """
+        Vectorized selection between two argument vectors.
+        if_else(True, 1, 2) > 1, if_else(False, 1, 2) -> 2.
+        numpy.where behavior: where(None, 1, 2) -> 2
+        """
+        # could check if x and y are compatible types
+        return self.__triop_expr__("where", x, y, method=True)
+
     def is_in(self, x):
         """
         Set membership (vectorized).
@@ -791,11 +844,13 @@ class Term(PreTerm, abc.ABC):
         """
         return self.__op_expr__("co_equalizer", x, inline=False, method=True)
 
-    def mapv(self, value_map, default_value):
+    def mapv(self, value_map, default_value=None):
         """
-        Map values (vectorized).
+        Map values to values (vectorized).
         """
         assert isinstance(value_map, DictTerm)
+        if default_value is None:
+            default_value = Value(None)
         assert isinstance(default_value, Value)
         return self.__triop_expr__(
             "mapv", x=value_map, y=default_value, inline=False, method=True
