@@ -26,7 +26,6 @@ methods_table = pd.DataFrame(
             ">=",
             "abs",
             "and",
-            "and",
             "arccos",
             "arccosh",
             "arcsin",
@@ -75,7 +74,6 @@ methods_table = pd.DataFrame(
             "mod",
             "month",
             "or",
-            "or",
             "parse_date",
             "parse_datetime",
             "quarter",
@@ -151,7 +149,6 @@ methods_table = pd.DataFrame(
             "x > y",
             "x >= y",
             "z.abs()",
-            "a & b",
             "a and b",
             "x.arccos()",
             "x.arccosh()",
@@ -201,7 +198,6 @@ methods_table = pd.DataFrame(
             "row_id.mod(2)",
             "date_col_0.month()",
             "a or b",
-            "a | b",
             "str_date_col.parse_date()",
             "str_datetime_col.parse_datetime()",
             "date_col_0.quarter()",
@@ -342,8 +338,6 @@ methods_table = pd.DataFrame(
             "e",
             "e",
             "e",
-            "e",
-            "e",
             "g",
             "g",
             "g",
@@ -508,8 +502,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
-            "y",
         ],
         "SQLiteModel": [
             "y",
@@ -535,7 +527,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "n",
             "y",
             "y",
@@ -579,7 +570,6 @@ methods_table = pd.DataFrame(
             "y",
             "n",
             "y",
-            "y",
             "n",
             "n",
             "n",
@@ -626,9 +616,9 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "n",
-            "n",
-            "n",
-            "n",
+            "w",
+            "y",
+            "y",
             "n",
             "y",
             "n",
@@ -656,7 +646,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "n",
             "n",
             "n",
@@ -721,7 +710,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "w",
             "n",
             "y",
@@ -752,9 +740,9 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "n",
-            "n",
-            "n",
-            "n",
+            "w",
+            "y",
+            "y",
             "n",
             "y",
             "n",
@@ -782,7 +770,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "n",
             "n",
             "n",
@@ -831,7 +818,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "n",
             "n",
             "y",
@@ -878,9 +864,9 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "n",
-            "n",
-            "n",
-            "n",
+            "w",
+            "y",
+            "y",
             "n",
             "y",
             "n",
@@ -908,7 +894,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "n",
             "n",
             "n",
@@ -957,7 +942,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "n",
             "n",
             "y",
@@ -1004,9 +988,9 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "n",
-            "n",
-            "n",
-            "n",
+            "w",
+            "y",
+            "y",
             "n",
             "y",
             "n",
@@ -1034,7 +1018,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "n",
             "n",
             "n",
@@ -1083,7 +1066,6 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "y",
-            "y",
             "n",
             "n",
             "y",
@@ -1130,9 +1112,9 @@ methods_table = pd.DataFrame(
             "y",
             "y",
             "n",
-            "n",
-            "n",
-            "n",
+            "w",
+            "y",
+            "y",
             "n",
             "y",
             "n",
@@ -1142,130 +1124,128 @@ methods_table = pd.DataFrame(
             "y",
         ],
         "version": [
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
-            "1.3.3",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
+            "1.3.4",
         ],
     }
 )
