@@ -214,15 +214,12 @@ def populate_impl_map(data_model) -> Dict[str, Callable]:
         "%": numpy.mod,
         "**": numpy.power,
         "and": _k_and,
-        "&": _k_and,
-        "&&": _k_and,
+        "&": numpy.bitwise_and,
         "or": _k_or,
         "|": _k_or,
-        "||": _k_or,
         "xor": numpy.logical_xor,
         "^": numpy.logical_xor,
-        "not": numpy.logical_not,
-        "!": numpy.logical_not,
+        "not": numpy.bitwise_or,
         "where": _where_expr,
         "if_else": _if_else_expr,
         "is_nan": data_model.isnan,
